from .base import Router

class CommunityRouter(Router):
    def __init__(self, node, **kwargs):
        super().__init__(node, **kwargs)

        self.community = self.node.network.community

    @property
    def local_community(self):
        links = self.node.links
        community = self.node.community
        return [met for met in links if met in community]

    @property
    def not_local_community(self):
        links = self.node.links
        community = self.node.community
        return [met for met in links if met not in community]

    @staticmethod
    def best(nodes, key):
        best = max(nodes, key=key)
        return best, key(best)

    def ui(self):
        return self.get_ui(self.node)

    def get_ui(self, x):
        return self.community.get_ui(x)

    def best_ui(self):
        return self.best(self.local_community, self.get_ui)

    def lp(self):
        return self.get_lp(self.node)

    def get_lp(self, x):
        return self.community.get_lp(x)

    def best_lp(self):
        return self.best(self.local_community, self.get_lp)

    def gp(self):
        return self.get_gp(self.node)

    def get_gp(self, x):
        return self.community.get_gp(x)

    def best_gp(self):
        return self.best(self.not_local_community, self.get_gp)

    def cbc(self, c_y):
        return self.get_cbc(self.node, c_y)

    def get_cbc(self, x, c_y):
        return self.community.get_cbc(x.community, c_y)

    def best_cbc(self, c_y):
        get_cbc = lambda x: self.get_cbc(x, c_y)
        return self.best(self.not_local_community, get_cbc)

    def ncf(self, c_y):
        return self.get_ncf(self.node, c_y)

    def get_ncf(self, x, c_y):
        return self.community.get_ncf(x, c_y)

    def best_ncf(self, c_y):
        get_ncf = lambda x: self.get_ncf(x, c_y)
        return self.best(self.local_community, get_ncf)

